/*
 *  Interactive disassembler (IDA).
 *  TheHyper VM Processor Module.
 *
 *  (c) 2006 Rolf Rolles
 */

#include "../idaidp.hpp"
#include "reg.hpp"
#include "TH.hpp"

inline void OutReg( int rgnum ) { out_register( ph.regNames[rgnum] ); }

void header  ( void ) { } // not implemented (yet)
void footer  ( void ) { } // not implemented (yet)
void segstart( ea_t ) { } // not implemented (yet)
void segend  ( ea_t ) { } // not implemented (yet)

void out( void )
{
    char buf[MAXSTR];
    
    init_output_buffer(buf, sizeof(buf));
    OutMnem();
    
    if( cmd.Op1.type != o_void )
        out_one_operand( 0 );
    
    if( cmd.Op2.type != o_void )
    {
        out_symbol( ',' );
        OutChar( ' ' );
        out_one_operand( 1 );
    }
    
    term_output_buffer();
    gl_comm = 1;                          // ask to attach a possible user-
                                          // defined comment to it
    MakeLine( buf );
}

bool outop( op_t &x )
{
  uval_t v;

  switch( x.type )
  {
  case o_imm:
      OutValue( x, OOFW_IMM );
      break;
  
  case o_phrase:
      out_symbol('[');
      OutReg( x.phrase );
      out_symbol(']');
      break;

  case o_displ:
      out_symbol('[');
      OutReg( x.phrase );
      out_symbol('+');
      OutValue(x, OOF_ADDR );
      out_symbol(']');
      break;
  
  case o_reg:
      OutReg( x.reg );
      break;
  
  case o_mem:
      out_symbol('[');
      OutValue(x, OOF_ADDR );
      out_symbol(']');
      break;
  
  case o_vmmem:
      if( !out_name_expr(x, get_keyed_address(x.addr), get_keyed_address(x.addr)) )
      { // if we could not create and output a name expression from the image of the keyed address
          OutValue(x, OOF_ADDR | OOF_NUMBER | OOFW_32); // instead output a raw value
          QueueMark(Q_noName, cmd.ea); // and mark this as a problem
	    }
      break;

  case o_far:
  case o_near:
      if( !out_name_expr(x, x.addr, x.addr) ) 
      { // if we could not create and output a name expression from the address
          OutValue(x, OOF_ADDR | OOF_NUMBER | OOFW_32); // see above
          QueueMark(Q_noName, cmd.ea);
	    }
      break;
  
  default:
      warning( "out: %lx: bad optype %d", cmd.ea, x.type );
  }

  return 1;
}
